<?php declare(strict_types=1);

namespace Plugin\km_rechnungen;

use JTL\Shop;
use JTL\Events\Dispatcher;
use JTL\License\Manager;
use JTL\Plugin\Bootstrapper;
use JTL\Plugin\State;
use JTL\Session\Frontend;
use JTL\Smarty\JTLSmarty;
use Plugin\km_rechnungen\classes\Shoprechnungen;
use stdClass;

require_once 'classes/class.shoprechnungen.php';

class Bootstrap extends Bootstrapper {
    private int $time;

    public function boot(Dispatcher $dispatcher) {
        parent::boot($dispatcher);
        $dispatcher->listen('shop.hook.'. HOOK_SMARTY_INC, function($args) {
           $this->execute($args);
        });

        $dispatcher->listen('shop.hook.'. HOOK_JTL_PAGE_MEINKKONTO, function($args) {
            $this->execute($args);
        });

        $this->customLicenseCheck();
        if(isset($_REQUEST['km_snackys_reset_licence'])) {
            $this->resetLicenceType();
        }
    }

    private function execute(array $args): void {
        $oPlugin        = $this->getPlugin();
        $Shoprechnungen = new Shoprechnungen(false, $oPlugin);
        $kDokument      = (int)($_REQUEST['kDokument'] ?? 0);
        $oSmarty        = $args['oSmarty'] ?? \Shop::Smarty();
        $nDisplayModus  = (int)($oPlugin->getConfig()->getAssoc()['km_shoprechnungen_display_modus'] ?? 1);

        if($kDokument) {
            $oDokument = $this->getDB()->select('km_shoprechnungen_document_mapping', 'kDokument', $kDokument);
            if($oDokument) {
                $file = PFAD_ROOT . 'plugins/' . $oPlugin->getPluginID() . '/' . $oDokument->cDateipfad;
                if(file_exists($file)) {
                    header_remove();
                    header("Content-type: application/pdf");
                    header("Content-Disposition: inline; filename=" .$oDokument->cName.".pdf");
                    @readfile($file);
                    die();
                }
            }
        }
        else {
            $nPageType = \Shop::getPageType();
            if(in_array($nPageType, [PAGE_MEINKONTO, PAGE_UNBEKANNT])) {
                $oSmarty->assign('erock_display_modus', $nDisplayModus);
                $oSmarty->assign('cTemplateName', strtolower(Shop::Container()->getTemplateService()->getActiveTemplate()->cName));
                $oSmarty->assign('shoprechnungen_icon', URL_SHOP . 'templates/Snackys/img/icons/icons.svg?v=5.1.5#icon-pdf');
                $oSmarty->assign('shoprechnungen_documents_link', URL_SHOP . 'Mein-Konto?shoprechnungen_documents=1');

                $oLangArr = $oPlugin->getLocalization()->getLangVars();
                foreach($oLangArr as $oLang) {
                    $oSmarty->assign($oLang->cName, $oLang->oPluginSprachvariableSprache_arr[strtoupper($_SESSION['cISOSprache'] ?? 'ger')]);
                }

                if (!isset($_REQUEST['bestellung']) && Frontend::getCustomer()->kKunde !== null) {
                    $oSmarty->assign('Dokumente', $Shoprechnungen->holeDokumente(Frontend::getCustomer()->kKunde));
                } else {
                    $oSmarty->assign('Dokumente', $Shoprechnungen->holeDokumenteViaBestellung((int)($_REQUEST['bestellung'] ?? 0)));
                }
            }
        }
    }

    public function updated($oldVersion, $newVersion) {
        parent::updated($oldVersion, $newVersion);
        //Check Licence Type > if from own Server remove EXS-ID, if not > add it!
        $this->resetLicenceType();
    }

    public function enabled() {
        parent::enabled();
        //Recheck Licence
        $this->resetLicenceType();
    }

    public function resetLicenceType() {
        $xml = simplexml_load_file($this->getPlugin()->getPaths()->getBasePath() . 'info.xml');
        if(!$xml)
            return; //abort because plugin info xml couldnt be loaded

        $obj = new stdClass;
        $obj->exsID = null;
        if(!empty($xml->ExsID))
        {
            //ExsID vorhanden, also in Tabelle einfügen, einfach damit es sicher ist wenn z.B. vorher ein eigener Lizenzserver verwendet wurde
            $obj->exsID = (string) $xml->ExsID;
        }

        //Datensatz in DB aktualisieren
        $this->getDB()->update('tplugin','cPluginID',$this->getPlugin()->getPluginID(),$obj);

        //Now Flush Cache Tags!
        $this->getCache()->flushTags([CACHING_GROUP_LICENSES]);
    }

    private function customLicenseCheck(): void {
        $license = $this->getPlugin()->getLicense()->getExsLicense();
        //Ist es eine Lizenz aus dem Extension Store?
        if($license != null)
        {
            //Wenn Licene abgelaufen: Auf Nova umstellen!
            if($license->getLicense()->isExpired())
            {
                $this->deactivatePlugin();
            }
        } else {
            //auf eigenem Server prüfen, sowie im Admin ein Info Tab anhängen!
            if($this->getCache()->get('km_lCheck_'.$this->getPlugin()->getPluginID()) < time()-(60*60*24))
                $this->checkLicenceByOwnServer();
        }
    }

    private function deactivatePlugin($message=false) {
        //Reset $time to zero to prevent next fast "reactivation"
        $this->time = 0;
        if($message)
            Shop::Container()->getLogService()->log(\JTLLOG_LEVEL_ERROR,$message,[0]);

        //Jetzt deaktivieren
        $this->getPlugin()->selfDestruct(State::ESX_LICENSE_EXPIRED, $this->getDB(), $this->getCache());

        //Jetzt noch alles aus der settings Tabelle entfernen!
        $this->getDB()->query('DELETE FROM km_plugin_settings WHERE `setting` LIKE \'%'.$this->getPlugin()->getPluginID().'%\'',3);

        Shop::Container()->getLogService()->log(\JTLLOG_LEVEL_ERROR,$this->getPlugin()->getPluginID().' wegen fehlender Lizenz deaktiviert',[0]);
    }

    public function checkLicenceByOwnServer($lastCheck = true): bool {
        $pluginId   = $this->getPlugin()->getPluginID();
        $version    = str_replace(".", "",(string) $this->getPlugin()->getCurrentVersion());
        $this->time = time();

        if($lastCheck === true) {
            //Prüfen ob schon geprüft wurde
            $lastCheckTime = $this->getDB()->query('SELECT value FROM km_plugin_settings WHERE `setting`=\'lCheck_'.$pluginId.'\'',1);
            if($lastCheckTime && $lastCheckTime->value > time()-(60*60*24))
                return true;
        }

        $shopurl = URL_SHOP;
        $request = array(
            'plugin' => $pluginId,
            'version' => $version,
            'shopurl' => $shopurl,
            'shopEmail' => '',
            'plugins' => array(0 => array(
                'plugin' => $pluginId,
                'version' => $version
            ))
        );


        //Jetzt Licence Request stellen
        $requestURL = 'https://licence.knoell-marketing.de/api2.php?'.http_build_query($request);
        $result     = file_get_contents($requestURL, false, stream_context_create(["http"=>["timeout"=>7]]));

        if(!$result) {
            $this->getDB()->query('INSERT INTO km_plugin_settings SET `setting`=\'lCheck_'.$pluginId.'\',`value`=\''.$this->time.'\' ON DUPLICATE KEY UPDATE `value`=\''.$this->time.'\'',3);	//To Avoid permanent checks
            return true;
        }

        //Remove Copyfree Data
        $this->getDB()->query('DELETE FROM km_plugin_settings WHERE `setting` LIKE \'km_snackys_copyfree%\'',3);

        $result = json_decode($result);
        if($result && isset($result->success) && $result->success) {
            foreach($result->jobs as $job) {
                if(isset($job->type) && $job->type == 'deactivate')
                    $this->deactivatePlugin($job->plugin,$job->message);
                elseif(is_array($job)) {
                    foreach($job as $item) {
                        if($item->type == 'update') {
                            $this->getDB()->query('INSERT INTO km_plugin_settings SET `setting`=\''.$item->setting.'\', `value`=\''.$item->value.'\'
								ON DUPLICATE KEY UPDATE `value`=\''.$item->value.'\'',3);
                            if(isset($item->message) && $item->message)
                                Shop::Container()->getLogService()->log(\JTLLOG_LEVEL_ERROR,$item->message,[0]);
                        }
                    }
                }
            }
        }

        $this->getDB()->query('INSERT INTO km_plugin_settings SET `setting`=\'lCheck_'.$pluginId.'\',`value`=\''.$this->time.'\' ON DUPLICATE KEY UPDATE `value`=\''.$this->time.'\'',3);
        $this->getCache()->set('km_lCheck_'.$this->getPlugin()->getPluginID(),$this->time,[CACHING_GROUP_TEMPLATE,CACHING_GROUP_PLUGIN]);
        return true;
    }

    public function renderAdminMenuTab(string $tabName,
                                       int $menuID,
                                       JTLSmarty $smarty
    ): string {
        if ($tabName !== 'Info') {
            return parent::renderAdminMenuTab($tabName, $menuID, $smarty);
        }

        $license = $this->getPlugin()->getLicense()->getExsLicense();

        //Ist es eine Lizenz aus dem Extension Store?
        if($license != null) {
            //License aus dem Extension store
            $herkunft = "JTL Extension Store";
            $typeVersion = "Siehe JTL Kundencenter";
            $neusteVersion = "Siehe JTL Kundencenter";
            $maxVersion = "Siehe JTL Kundencenter";
            $gueltigBis = "Siehe JTL Kundencenter";

        } else {
            //auf eigenem Server prüfen, sowie im Admin ein Info Tab anhängen!
            if(isset($_REQUEST['resetLicence']))
                $this->checkLicenceByOwnServer(false);

            $pluginId   = $this->getPlugin()->getPluginID();
            $herkunft   = "Knoell Marketing Shop";
            $max        = $this->getDB()->select('km_plugin_settings','setting',$pluginId.'_max');
            $neuste     = $this->getDB()->select('km_plugin_settings','setting',$pluginId.'_neuste');
            $type       = $this->getDB()->select('km_plugin_settings','setting',$pluginId.'_type');
            $gueltig    = $this->getDB()->select('km_plugin_settings','setting',$pluginId.'_gueltig');

            $typeVersion = 'unknown';
            if($type)
                switch($type->value) {
                    case 'test':
                        $typeVersion = 'Testversion';
                        break;
                    case 'live':
                        $typeVersion = 'Liveversion';
                        break;
                    case 'abo':
                        $typeVersion = 'Abonnement';
                        break;
                    case 'dev':
                        $typeVersion = 'Developer';
                        break;
                }

            $neusteVersion = $this->getPlugin()->getCurrentVersion();
            if($neuste)
                $neusteVersion = substr($neuste->value,0,1).'.'.substr($neuste->value,1,1).'.'.substr($neuste->value,2,1);

            $maxVersion = 'unknown';
            if($max)
                $maxVersion = substr($neuste->value,0,1).'.'.substr($max->value,1,1).'.'.substr($max->value,2,1);

            $gueltigBis = 'unknown';
            if($gueltig)
                $gueltigBis = date('d.m.Y',intval($gueltig->value));

        }

        $copyfree = false;
        $lManager = new Manager($this->getDB(),$this->getCache());
        $copyfree = $lManager->getLicenseByExsID('ee615c29-7509-41c9-af4a-2930675a16b2');	//km_snackys_copyfree > inApp Licence
        if(!$copyfree || $copyfree->getLicense()->isExpired()) {
            $check = \Shop::Container()->getDB()->query('SELECT value FROM km_plugin_settings WHERE `setting`=\'km_snackys_copyfree_gueltig\'',1);
            $copyfree = (!$check) ? 0:$check->value;
        } else {
            $copyfree = true;
        }

        $code = '<div class="panel panel-default first">
			<div class="panel-heading">
			<h3 class="panel-title">Über das Plugin und Ihre Version</h3></div>
			<div class="form-group form-row align-items-center">
			    <div class="col-md-3"><label for="km_snackys_sale_prozent">Lizenzherkunft</label></div>
				<div class="col-md-3"><input type="text" class="form-control" value="'.$herkunft.'" style="background:#fff;text-align:right;" readonly></div>
			</div>
			<div class="form-group form-row align-items-center">
			    <div class="col-md-3"><label for="km_snackys_sale_prozent">Lizenztyp</label></div>
			    <div class="col-md-3"><input type="text" readonly="" class="form-control" value="'.$typeVersion.'" style="background:#fff;text-align:right;"></div>
			</div>
			<div class="form-group form-row align-items-center">
			    <div class="col-md-3"><label for="km_snackys_sale_prozent">Aktuell installierte Version</label></div>
			    <div class="col-md-3"><input type="text" readonly="" class="form-control" value="'.$this->getPlugin()->getCurrentVersion().'" style="background:#fff;text-align:right;"></div>
			</div>
			<div class="form-group form-row align-items-center">
			    <div class="col-md-3"><label for="km_snackys_sale_prozent">Neuste Version</label></div>
			    <div class="col-md-3"><input type="text" readonly="" class="form-control" value="'.$neusteVersion.'" style="background:#fff;text-align:right;"></div>
			</div>
			<div class="form-group form-row align-items-center">
				<div class="col-md-3"><label for="km_snackys_sale_prozent">Ihre maximale Version</label></div>
                <div class="col-md-3"><input type="text" readonly="" class="form-control" value="'.$maxVersion.'" style="background:#fff;text-align:right;"></div>
			</div>
			<div class="form-group form-row align-items-center">
			    <div class="col-md-3"><label for="km_snackys_sale_prozent">Upgrades verfügbar bis</label></div>
			    <div class="col-md-3"><input type="text" readonly="" class="form-control text" value="'.$gueltigBis.'" style="background:#fff;text-align:right;"></div>
			</div>
			<div class="form-group form-row align-items-center">
				<div class="input-group">
			        <form method="post" action="" class="navbar-form"><button name="resetLicence" type="submit" value="Speichern" class="btn btn-primary"><i class="fa fa-save"></i> Lizenz neu prüfen</button></form>
				</div>
			</div>
		</div>';
        return $code;
    }
}